DEPS ?= $(CURDIR)/deps

DIALYZER_OPTS ?= -Wunderspecs

# FIXME: on first run you won't have your deps and this is empty.

# Find all the deps the project has by searching the deps dir
ALL_DEPS = $(notdir $(wildcard deps/*))
# Create a list of deps that should be used by dialyzer by doing a
# complement on the sets
DEPS_LIST = $(filter-out $(DIALYZER_SKIP_DEPS), $(ALL_DEPS))
# Create the path structure from the dep names
# so dialyzer can find the .beam files in the ebin dir
# This list is then used by dialyzer in creating the local PLT
DIALYZER_DEPS = $(foreach dep,$(DEPS_LIST),deps/$(dep)/ebin)

DEPS_PLT = deps.plt

ERLANG_DIALYZER_APPS = asn1 \
                       compiler \
                       crypto \
                       edoc \
                       edoc \
                       erts \
                       eunit \
                       eunit \
                       gs \
                       hipe \
                       inets \
                       kernel \
                       mnesia \
                       mnesia \
                       observer \
                       public_key \
                       runtime_tools \
                       runtime_tools \
                       ssl \
                       stdlib \
                       syntax_tools \
                       syntax_tools \
                       tools \
                       webtool \
                       xmerl

all: .concrete/DEV_MODE compile eunit dialyzer $(ALL_HOOK)

.concrete/DEV_MODE:
	@mkdir -p .concrete
	@touch $@

# Clean ebin and .eunit of this project
clean:
	@rebar clean skip_deps=true
	@rm -rf ebin_dialyzer

# Clean this project and all deps
allclean:
	@rebar clean

compile: $(DEPS)
	@rebar compile

$(DEPS):
	@rebar get-deps

# Full clean and removal of all deps. Remove deps first to avoid
# wasted effort of cleaning deps before nuking them.
distclean:
	@rm -rf deps $(DEPS_PLT)
	@rebar clean

eunit:
	@rebar skip_deps=true eunit

test: eunit

# There is no easy way to filter out warnings for uncooperative
# modules. Here we provide an ugly hack that allows us to regularly
# run Dialyzer and ignore the warnings that come from neotoma
# generated code. We run dialyzer against the ebin_dialyzer directory
# which is created anew for each run and has the problem modules
# (those generated by neotoma) removed.
dialyzer: ~/.dialyzer_plt $(DEPS_PLT)
	@rm -rf ebin_dialyzer
	@mkdir -p ebin_dialyzer
	@cp ebin/* ebin_dialyzer
	@rm ebin_dialyzer/*lucene*.beam
	@dialyzer $(DIALYZER_OPTS) --plts ~/.dialyzer_plt $(DEPS_PLT) -r ebin_dialyzer

$(DEPS_PLT):
	- @dialyzer --build_plt $(DIALYZER_DEPS) --output_plt $(DEPS_PLT)

~/.dialyzer_plt:
	@echo "ERROR: Missing ~/.dialyzer_plt. Please wait while a new PLT is compiled."
	dialyzer --build_plt --apps $(ERLANG_DIALYZER_APPS)
	@echo "now try your build again"

doc:
	@rebar doc skip_deps=true

tags:
	find src deps -name "*.[he]rl" -print | etags -

.PHONY: all compile eunit test dialyzer clean allclean distclean doc tags
